//------------------------------------------------------------------------------
// <copyright>
//   Copyright (c) 2006 Nesterovsky bros. All rights reserved.
//   Home: http://www.nesterovsky-bros.com
//   E-mail: mailto:contact@nesterovsky-bros.com
// </copyright>
// <description>
//   Custom functoids for BizTalk Server 2006.
// </description>
// <history>
//  Created: 15 Jun 2006, Arthur & Vladimir Nesterovsky
// </history>
//------------------------------------------------------------------------------

namespace NesterovskyBros.BizTalk.Functoids
{
	using System;
	using System.Reflection;
	using System.Text;
	using System.Collections;
	using System.Globalization;
	using System.Text.RegularExpressions;
	using Microsoft.BizTalk.BaseFunctoids;

	/// <summary>
	/// LogicalToVisualFunctoid converts visual encoded string to logical 
	/// encoded one.
	/// </summary>
	public class LogicalToVisualFunctoid: BaseFunctoid
	{
		public LogicalToVisualFunctoid()
		{
			//ID for this functoid
      ID = 0x415602;

			// resource assembly must be ProjectName.ResourceName if building 
			// with VS.Net
			SetupResourceAssembly("NesterovskyBros.BizTalk.Functoids.Resources", 
				Assembly.GetExecutingAssembly());

			// setup the Name, ToolTip, Help Description, and the Bitmap for 
			// this functoid
			SetName("LogicalToVisualFunctoid_Name");
			SetDescription("LogicalToVisualFunctoid_Description");
			SetTooltip("LogicalToVisualFunctoid_ToolTip");
			SetBitmap("LogicalToVisualFunctoid_Bitmap");

			// set min and max parameters counters
			SetMinParams(1);
			SetMaxParams(2);

			// set the function name that needs to be called when this Functoid
			// is invoked. This means that this Functoid assembly need to be present 
			// in GAC for its availability during Test, Map and Runtime.
			SetExternalFunctionName(GetType().Assembly.FullName,
				"NesterovskyBros.BizTalk.Functoids.LogicalToVisualFunctoid", "Apply");

			// specify a category for this functoid. This functoid goes under the 
			// String Functoid Tab in the VS.Net toolbox for functoids.
			Category = FunctoidCategory.String;

			// specify input and output parameters types
			AddInputConnectionType(ConnectionType.AllExceptRecord);
			AddInputConnectionType(ConnectionType.AllExceptRecord);

			OutputConnectionType = ConnectionType.AllExceptRecord;
		}

		/// <summary>
		/// Implements the logical to visual conversion.
		/// </summary>
		/// <param name="source">determines a source string.</param>
		/// <param name="baseEmbeddingLevel">
		/// determines a left-to-right ("ltr") or right-to-left ("rtl") dirrection 
		/// of source string.
		/// </param>
		/// <returns>string in visual encoding.</returns>
    public string Apply(string source, string baseEmbeddingLevel)
		{
      bool rtl = (baseEmbeddingLevel != null) && 
        string.Compare(baseEmbeddingLevel, "rtl", true) == 0;

			return NesterovskyBros.VisualConverters.VisualConverter.DoConvert(source, rtl,
				true);
    }
	}
}