// This is the main DLL file.

#include "stdafx.h"
#include <Usp10.h>
#include <vcclr.h>

#include "LogicalToVisualConverter.h"

namespace NesterovskyBros
{
  namespace VisualConverters
  {

// Utility class to keep arrays.
template<class T>
class array
{
public:
  explicit array(T* anArray): a(anArray) { }
  explicit array(int count): a(new T[count]) { }
  ~array() { delete[] a; }

  operator T* () { return a; }
  T& operator[] (int index) { return a[index]; }
private:
  T* a;
};

String^ VisualConverter::DoConvert(String^ value, bool rtl, bool convertDirection)
{
  if (value == nullptr)
  {
    return nullptr;
  }

  int count = value->Length;

  if (count <= 1)
    return value;

  cli::pin_ptr<const wchar_t> chars = PtrToStringChars(value);

  if (ScriptIsComplex(chars, count, SIC_COMPLEX | SIC_NEUTRAL) != S_OK)
  {
    return value;
  }

  SCRIPT_CONTROL control = SCRIPT_CONTROL();
  SCRIPT_STATE state = SCRIPT_STATE();
  int resultCount = 0;
  int maxCount = count + 1;

 // control.fInvertPreBoundDir = rtl ? 1 : 0;
  state.uBidiLevel = rtl ? 1 : 0;

  array<SCRIPT_ITEM> items(maxCount);

  if (ScriptItemize(chars, count, maxCount, &control, &state, items, &resultCount) != 0)
  {
    return nullptr;
  }

  array<BYTE> levels(count);

  for(int i = 0; i < resultCount; i++)
  {
    SCRIPT_ITEM &item = items[i];
    char level = item.a.s.uBidiLevel;

    for(int j = item.iCharPos, k = items[i + 1].iCharPos; j < k; j++)
    {
      levels[j] = level;
    }
  }

  array<int> orders(count);

  if (ScriptLayout(count, levels, orders, 0) != 0)
  {
    return nullptr;
  }

  array<wchar_t> result(count);

  for(int i = 0; i < count; i++)
  {
    result[i] = chars[orders[i]];
  }

  return gcnew String(result, 0, count);
}

  } // namespace VisualConverters
} // namespace NesterovskyBros